<?php
/**
 * Partner API Library
 * 
 * @copyright Copyright (c) 2020 Asseco Data Systems SA
 * @license license.txt
 */

/*
 * Zalecamy przeczytanie w całości tego pliku.
 */

/*
 * Na początku należy dołączyć plik zawierający klasę usługi.
 */

require_once 'certumPartnerAPI/service.php';

/*
 * Utworzenie obiektu usługi.
 * 
 * Argument trzeci i czwarty został pominięty w konstruktorze.
 * Zostaną przypisane im wartości domyślne, które oznaczają, że zostanie
 * użyty adres testowej usługi oraz język angielski.
 */

$service = new PartnerAPIService('userName', 'password');

/*
 * Aby użyć usługi w środowisku produkcyjnym, należy podać jej adres jako
 * trzeci argument. Można również podać jawnie język w czwartym argumencie.
 * 
 * W pozostałych plikach przykładów używany jest domyślny adres testowy.
 */

$service = new PartnerAPIService('userName', 'password', PartnerAPIService::WSDL_PROD, 'pl');

/*
 * Opcjonalnie, można zdefiniować plik, który posłuży do zapisu danych
 * pomocnych przy analizie zapytań do usługi.
 */

$service->setDebugFile('/tmp/debug.txt');

/*
 * Opcjonalnie obiekt usługi może być skonfigurowany tak, aby przechwytywał
 * wyjątki SoapFault i rzucał je ponownie jako wyjątki PartnerAPIException.
 * 
 * Należy pamiętać, że w niektórych przypadkach, pomimo przechwycenia wyjątku
 * SoapFault, PHP i tak wypisuje komunikat o błędzie na standardowe wyjście
 * błędów (stderr).
 */

$service->setCatchSoapFault(TRUE);

/*
 * Teraz obiekt usługi jest skonfigurowany i gotowy do pracy.
 * Taki obiekt jest używany w pozostałych plikach przykładów.
 * 
 * Cała biblioteka jest napisana tak, aby wszędzie utrzymać płynny interfejs,
 * tzn. że część metod zwraca ten sam obiekt, na którym wywołano metodę.
 * Sposób użycia metod zależy od preferencji użytkownika.
 * Metoda call() w każdej operacji zwraca wartość logiczną TRUE lub FALSE.
 * Dla każdej operacji część parametrów może być obowiązkowa a część
 * opcjonalna (patrz: dokumentacja usługi webowej w PDF).
 * Operacje i metody można wołać w ten sposób:
 */

$op = $service->operationXXX();
$op->setYYY('yyy');
$op->setZZZ('zzz');
$res = $op->call();
if ($res) {
    // sukces
} else {
    // błąd operacji
}
// lub
$op->call();
if ($op->isSuccess()) {
    // sukces
} else {
    // błąd operacji
}

/*
 * oraz w ten:
 */

if ($service->operationXXX()->setYYY('yyy')->setZZZ('zzz')->call()) {
    // sukces
} else {
    // błąd operacji
}

/*
 * Biblioteka może rzucić wyjątek PartnerAPIException.
 * W zależności od konfiguracji omówionej powyżej, może też zostać rzucony
 * wyjatek SoapFault.
 * Dlatego działania na obiektach biblioteki należy zawrzeć w wyrażeniu
 * try {} catch {}
 */

try {
    $op = $service->operationXXX();
    $op->setYYY('yyy');
    if ($op->call()) {
        // sukces
    } else {
        // błąd operacji
    }
} catch (PartnerAPIException $e) {
    // obsługa wyjątku
}

/*
 * Każdy obiekt operacji ma kilka dodatkowych przydatnych metod:
 */

/*
 * getResponseTimeLocal - zwraca czas odpowiedzi przekonwertowany na
 * czas lokalny:
 */

$op->getResponseTimeLocal();

/*
 * getInputDataAsArray - zwraca tablicę ze wszystkimi danymi wejściowymi
 * operacji, argument ustawiony na TRUE oznacza, że w tablicy nie znajdą się
 * elementy, które nie zostały ustawione przez użytkownika:
 */

$op->getInputDataAsArray();
$op->getInputDataAsArray(true);

/*
 * getOutputDataAsArray - zwraca tablicę ze wszystkimi danymi wyjściowymi
 * z operacji:
 */

$op->getOutputDataAsArray();

/*
 * getResponseMessage - zwraca obiekt reprezentujący całą wiadomość
 * uzyskaną w odpowiedzi na żądanie
 */

$op->getResponseMessage();

/*
 * getResponseHeader - zwraca nagłówek zawarty w odpowiedzi. W nagłowku tym
 * znajdują się (choć nie zawsze są wypełniane) pola dotyczące ilości
 * rekordów i stronicowania, czas odpowiedzi oraz błędy operacji:
 */

$header = $op->getResponseHeader();
print $header->currentPage;
print $header->pagesCount;
print $header->returnCount;
print $header->successCode;
print $header->timestamp;

/*
 * Obsługa błędów w nagłówku powinna wyglądać jak poniżej.
 * Błędy nie zawsze występują, więc najczęściej to pole będzie zawierało
 * wartość NULL. Trzeba zatem to sprawdzać:
 */

if (! is_null($header->errors)) {
    $error = $header->errors->Error;
    if (is_array($error)) {
        foreach ($error as $e) {
            print $e->errorCode;
            print $e->value;
        }
    } else {
        print $error->errorCode;
        print $error->value;
    }
}

/*
 * Można też zrobić to w prostszy sposób:
 */

$errors = $op->getErrors();
foreach ($errors as $error) {
    print $error->errorCode;
    print $error->value;
}

/*
 * Można też pobrać opisy błędów:
 */

$texts = $op->getErrorTexts();
foreach ($texts as $text) {
    print $text['code'];
    print $text['number'];
    print $text['text'];
}
